

class PluginBase(object):

    def __init__(self, name):
        self.name = name
    
    def __repr__(self):
        return '<Plugin %s>' % self.name
    
    def set_build(self, build):
        self.build = build

    def on_build_complete(self):
        pass



from ..base import BuildStage
from pathlib import Path


MANAGE_PY = """#!/usr/bin/env python
import os
import sys

if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "{0}.settings")
    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc
    execute_from_command_line(sys.argv)
"""


class FirstStage(BuildStage):
    def run(self):
        root = Path(self.build.build_path)
        manage_py = root / 'manage.py'
        manage_py.touch(0o755)
        contents = MANAGE_PY.format(self.build.project.name)
        manage_py.write_text(contents)


class AppsPackages(BuildStage):
    def run(self):
        root = Path(self.build.build_path)
        for app in self.build.details['apps']:
            if not app['external']:
                app_pkg = root / app['name']
                app_pkg.mkdir()
                (app_pkg / '__init__.py').touch()



from ..base import BuildStage
from pathlib import Path


class RequirementsStage(BuildStage):
    def run(self):
        req_txt_file = Path(self.build.build_path) / 'requirements.txt'
        requirements = set(['Django', 'Pillow'])
        for plugin in self.build.details['plugins']:
            for req in plugin.get('requirements', []):
                requirements.add(req)

        lines = '\n'.join(sorted(requirements)) + '\n'
        req_txt_file.write_text(lines)



import logging
import traceback
from datetime import datetime


class LogHandler(logging.Handler):
    def __init__(self, logger):
        super(LogHandler, self).__init__()
        self.logger = logger
        self.start_ts = 0
        
    def emit(self, record):
        if self.start_ts == 0:
            self.start_ts = record.created
        print ('%.5f %s' % (record.created - self.start_ts, record.msg))
        self.logger.records.append(record)


class Logger(logging.Logger):
    def __init__(self):
        super(Logger, self).__init__('prototyper', logging.DEBUG)
        self.records = []
        self.addHandler(LogHandler(self))
    
    def serialize(self):
        result = []
        for rec in self.records:
            result.append({
                'time': str(datetime.fromtimestamp(rec.created).time()),#.strftime('%H:%m:%s'),
                'level': rec.levelname,
                'message': str(rec.msg),
            })
        return result


def get_logger():
    return Logger()



from ..base import BuildStage
from pathlib import Path


class codelines(list):
    def __init__(self, initial=None):
        super(codelines, self).__init__(initial or [])
    
    def extend_indent(self, lines, indent=1):
        spaces = ' ' * (4 * indent)
        for l in lines:
            self.append(spaces + l)


def camel_to_spaces(s):
    import re
    return re.sub("([a-z])([A-Z])", "\g<1> \g<2>", s)


def code_string(s):
    q = "'" in s and '"' or "'"
    return q + s + q


class ModelsStage(BuildStage):
    def run(self):
        self.use_ugettext = self.build.details['build_settings'].get('ugettext_lazy', True)
        for app in self.build.details['apps']:
            if not app['external']:
                self._handle_app(app)
    
    def _handle_app(self, app):
        mdodels_py = Path(self.build.build_path) / app['name'] / 'models.py'
        contents = ['from django.db import models']
        if self.use_ugettext:
            contents.append('from django.utils.translation import ugettext_lazy as _')
        
        self._inheritance_imports(app, contents)

        for model in app['models']:
            contents.extend(['', ''])
            model_lines = ModelBuilder(self, app, model)
            contents.extend(model_lines)
        
        contents.append('')  # last empty line

        # self.log('\n'.join(contents))

        mdodels_py.write_text('\n'.join(contents))
    
    def _inheritance_imports(self, app, contents):
        all_classes = set()
        for model in app['models']:
            for m in model.get('inheritance', []):
                all_classes.add(m)
        for cls in sorted(all_classes):
            app, model = cls.split('.')
            contents.append('from %s.models import %s' % (app, model))


class ModelBuilder(codelines):
    def __init__(self, stage, app, model):
        super(ModelBuilder, self).__init__()
        self.stage, self.app, self.model = stage, app, model
        self._create()

    def _create(self):
        self.append('class %s(%s):' % (self.model['name'], self._inheritance()))

        for field in self.model['fields']:
            self.extend_indent(self._handle_field(field))

        self.extend_indent(self._handle_meta())
        
        if len(self) == 1:
            self.append('    pass')
    
    def _inheritance(self):
        classes = self.model.get('inheritance', [])
        if not classes:
            return 'models.Model'
        return ', '.join([i.split('.')[-1] for i in classes])
    
    def _handle_field(self, field):
        line = FieldBuilder(self, field).render()
        return [line]

    def _handle_meta(self):
        lines = codelines([''])
        lines.append('class Meta:')

        v_name = camel_to_spaces(self.model['name']).lower()
        v_name_plural = v_name
        if v_name_plural.endswith('y'):
            v_name_plural = v_name_plural[:-1] + 'ies'
        elif not v_name_plural.endswith('s'):
            v_name_plural += 's'
        lines.extend_indent(["verbose_name = %s" % self._trans_str(v_name)])
        lines.extend_indent(["verbose_name_plural = %s" % self._trans_str(v_name_plural)])
        
        return lines
    
    def _trans_str(self, s):
        "Returns either _('<s>' or '<s>' based on build settings)"
        result = code_string(s)
        if self.stage.use_ugettext:
            return '_(' + result + ')'
        return result


ONLY_COMMON_ATTRS = [
    'AutoField', 'BigAutoField', 'BigIntegerField',
    'BinaryField', 'BooleanField',
    'DurationField', 'EmailField', 'FloatField',
    'IntegerField', 'NullBooleanField',
    'PositiveIntegerField', 'PositiveSmallIntegerField',
    'SlugField', 'SmallIntegerField',
    'TextField', 'URLField', 'UUIDField',
]


class FieldBuilder(object):
    def __init__(self, model_builder, field):
        self.model_builder = model_builder
        self.name = field['name']
        self.type = field['type']
        self.attrs = field['attrs']
        self.relation = field['relation']
    
    def render(self):
        if self.type in ONLY_COMMON_ATTRS:
            attrs = self._common_attrs()
        else:
            method = getattr(self, '_attrs_%s' % self.type)
            attrs = method()
        return '%s = models.%s(%s)' % (self.name, self.type, ', '.join(attrs))
    
    def _common_attrs(self, verbose_name_kv=False):
        attributes = []

        _trans = self.model_builder._trans_str

        verbose_name = _trans(self.name.replace('_', ' '))
        if verbose_name_kv:
            verbose_name = 'verbose_name=' + verbose_name

        attributes.append(verbose_name)

        if self.attrs.get('primary_key') is True:
            attributes.append('primary_key=True')
        if self.attrs.get('null') is True:
            attributes.append('null=True')
        if self.attrs.get('blank') is True:
            attributes.append('blank=True')
        if self.attrs.get('unique') is True:
            attributes.append('unique=True')
        if self.attrs.get('db_index') is True:
            attributes.append('db_index=True')
        if self.attrs.get('editable') is False:
            attributes.append('editable=False')
        if self.attrs.get('default') is True:
            val = code_string(self.attrs['default'])
            if self.type in ('BigIntegerField', 'BooleanField', 'FloatField', 'IntegerField', 'IntegerField', 'NullBooleanField', 'PositiveIntegerField', 'PositiveSmallIntegerField', 'SmallIntegerField'):
                val = self.attrs['default']
            attributes.append('default=%s' % val)
        if self.attrs.get('help_text'):
            attributes.append('help_text=%s' % _trans(self.attrs['help_text']))
        return attributes
    
    def _attrs_CharField(self):
        build_settings = self.model_builder.stage.build.details['build_settings']
        attrs = self._common_attrs()
        if self.attrs.get('max_length'):
            max_length = self.attrs['max_length']
        else:
            max_length = build_settings.get('charfield_max_length', 200)
        attrs.append('max_length=%s' % max_length)
        return attrs
    
    def _relational(self):
        attrs = [code_string(self.relation)]
        attrs.extend(self._common_attrs(verbose_name_kv=True))
        if self.type in ('OneToOneField', 'ForeignKey'):
            attrs.append('on_delete=models.CASCADE')
        return attrs
    
    _attrs_ManyToManyField = _relational
    _attrs_OneToOneField = _relational
    _attrs_ForeignKey = _relational

    def _attrs_DecimalField(self):
        attrs = self._common_attrs()
        # TODO: maybe build settings ?
        attrs.extend(['max_digits=10', 'decimal_places=2'])
        return attrs
    
    def _attrs_DateTimeField(self):
        attrs = self._common_attrs()
        if self.attrs.get('auto_now') is True:
            attrs.append('auto_now=True')
        if self.attrs.get('auto_now_add') is True:
            attrs.append('auto_now_add=True')
        return attrs
    
    _attrs_DateField = _attrs_DateTimeField
    _attrs_TimeField = _attrs_DateTimeField

    def _attrs_FileField(self):
        attrs = self._common_attrs()
        if self.attrs.get('upload_to'):
            attrs.append('upload_to=%s' % code_string(self.attrs['upload_to']))
        return attrs

    _attrs_ImageField = _attrs_FileField

    def _attrs_FilePathField(self):
        # TODO
        return self._common_attrs()
    
    def _attrs_GenericIPAddressField(self):
        # TODO
        return self._common_attrs()



import os
from django.urls import path, re_path
from django.conf import settings
from django.views.static import serve
from .views import main_view, api_build, api_save, discover_plugins, install_plugin


urlpatterns = (
    path('', main_view),
    path('api/build/', api_build),
    path('api/save/', api_save),
    path('api/plugin/', discover_plugins),
    path('api/plugin/install/', install_plugin),

    re_path(r'^static/(?P<path>.*)$', serve, {'document_root': os.path.join(settings.BASE_DIR, 'static')}),
)



import os
import json
from django.conf import settings
from django.template import Template, Context
from django.http import JsonResponse, HttpResponse
from .build import run_build
from . import plugins

HOME_TEMPLATE = """<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <title>prototyper</title>
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
    <link href="https://use.fontawesome.com/releases/v5.0.7/css/all.css" rel="preload" as="style">
    <link href="https://use.fontawesome.com/releases/v5.0.7/css/all.css" rel="stylesheet">
</head>
<body>
    <div id="prototyper"></div>
    
    <script>
        var PROJECT_DATA = {{ PROJECT_DATA|safe }}
    </script>
    <script type="text/javascript" src="{{JS_BUNDLE}}"></script>
</body>
</html>
"""


def _js_bundle():
    if settings.DEV_MODE and not os.environ.get('STATIC_BUNDLE'):
        return 'http://localhost:9000/dist/build.js'
    return '/static/build.js'


def main_view(request):
    data = settings.PROTOTYPER_PROJECT.load()
    ctx = {
        'PROJECT_DATA': json.dumps(data),
        'JS_BUNDLE': _js_bundle(),
    }
    html = Template(HOME_TEMPLATE).render(Context(ctx))
    return HttpResponse(html)


def api_build(request):
    build = run_build()
    return JsonResponse({
        'success': build.success,
        'logs': build.logger.serialize()
    })


def api_save(request):
    data = _json_body(request)
    settings.PROTOTYPER_PROJECT.save(data)
    for a in data['apps']:
        print('%20s' % a['name'], ':', [m['name'] for m in a['models']])
    return JsonResponse({'success': True})


def discover_plugins(request):
    query = request.GET['q']
    data = plugins.search_plugins(query)
    return JsonResponse({'success': True, 'results': data})


def install_plugin(request):
    data = _json_body(request)
    plugin = plugins.install(data['name'], data['url'])
    return JsonResponse({'success': True, 'plugin': plugin})


def _json_body(request):
    return json.loads(request.body.decode('utf-8'))



from ..base import BuildStage
from pathlib import Path

TPL = """\"\"\"
WSGI config for {0} project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/2.0/howto/deployment/wsgi/
\"\"\"

import os

from django.core.wsgi import get_wsgi_application

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "{0}.settings")

application = get_wsgi_application()
"""


class WsgiStage(BuildStage):
    def run(self):
        wsgi_py = Path(self.build.settings_pckg_path) / 'wsgi.py'
        wsgi_py.write_text(TPL.format(self.build.project.name))



import os
import traceback
import tempfile
import shutil
from .log import get_logger


class Build(object):
    def __init__(self, project):
        self.details = project.load()
        self.project = project
        self.build_path = tempfile.mkdtemp(prefix='djprototyper')
        self.settings_pckg_path = os.path.join(self.build_path, project.name)  # TODO: can be configurable
        self.logger = get_logger()
        self.success = False  # succesful build finished
    
    def log(self, message):
        self.logger.info(message)
    
    def save(self):
        result_path = os.path.join(self.project.path, self.project.name)
        assert len(result_path) > 3  # just in case making sure no root folder is ever removed :)
        if os.path.exists(result_path):
            self.log('Cleaning previous build {}'.format(result_path))
            shutil.rmtree(result_path)
        self.log('Saving to "{}"'.format(result_path))
        shutil.move(self.build_path, result_path)

    def cleanup(self):
        if os.path.exists(self.build_path):
            self.log('Cleaning: {}'.format(self.build_path))
            shutil.rmtree(self.build_path)


class BuildStage(object):
    def __init__(self, build):
        self.build = build
    
    def run(self):
        raise NotImplementedError('please implement run')
    
    def log(self, message):
        self.build.log(message)


def pipeline(build, plugins, stages):
    for cls in stages:
        assert issubclass(cls, BuildStage)
        try:
            build.log('Running ' + cls.__name__)
            stage = cls(build)
            stage.run()
        except Exception as e:
            build.logger.error(traceback.format_exc())
            return False
    
    for plug in plugins:
        plug.on_build_complete()

    build.save()
    build.success = True
    return True



import os
import django
from django.conf import settings


def django_configure():
    BASE_DIR = os.path.dirname(__file__)

    SECRET_KEY = os.environ.get('SECRET_KEY', '{{ secret_key }}')
    # ALLOWED_HOSTS = os.environ.get('ALLOWED_HOSTS', 'localhost').split(',')
    ALLOWED_HOSTS = ['*']

    settings.configure(
        DEBUG=True,
        BASE_DIR=BASE_DIR,
        SECRET_KEY=SECRET_KEY,
        ALLOWED_HOSTS=ALLOWED_HOSTS,
        ROOT_URLCONF='prototyper.urls',
        INSTALLED_APPS=[
        ],
        TEMPLATES=[{
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
        }],
        MIDDLEWARE_CLASSES=(
            'django.middleware.common.CommonMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.middleware.clickjacking.XFrameOptionsMiddleware',
        ),

        DEV_MODE=os.environ.get('PROTOTYPER_DEV', 'no') == 'yes'
    )


def run_server():
    from django.core.management import call_command
    call_command('runserver', '0.0.0.0:8000')  # TODO: get from args



from pathlib import Path

from ..base import BuildStage


class AdminStage(BuildStage):
    def run(self):
        for app in self.build.details['apps']:
            if app['external'] is False:
                self._handle_app(app)

    def _handle_app(self, app):
        contents = ['from django.contrib import admin', 'from .models import *']

        for model in app['models']:
            if model['admin']['generate']:
                model['admin'].pop('generate')
                contents.extend(['', ''])
                contents.extend(self._handle_model(app, model))

        contents.append('')  # empty line

        admin_py = Path(self.build.build_path) / app['name'] / 'admin.py'
        admin_py.write_text('\n'.join(contents))

    def _handle_model(self, app, model):
        admin = [f"@admin.register({model['name']})", f"class {model['name']}Admin(admin.ModelAdmin):"]
        if model['admin']:
            for name, attr in model['admin'].items():
                if attr['fields']:
                    if attr['single']:
                        admin.append(f"  {name} = '{attr['fields'][0]}'")
                    else:
                        fields = [f'"{field}"' for field in attr['fields']]
                        admin.append(f"  {name} = [{', '.join(fields)}]")
        else:
            admin.append("  pass")
        return admin



from .core import FirstStage, AppsPackages
from .settings import SettingsStage
from .urls import UrlsStage
from .wsgi_app import WsgiStage
from .admin import AdminStage
from .models import ModelsStage
from .requirements import RequirementsStage



from ..base import BuildStage
from pathlib import Path


TPL = """\"\"\"
Django settings for {project_name} project.

Generated by 'django-admin startproject' using Django 2.0.

For more information on this file, see
https://docs.djangoproject.com/en/2.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.0/ref/settings/
\"\"\"

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '{SECRET_KEY}'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    {INSTALLED_APPS}
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = '{url_conf}'

TEMPLATES = [
    {{
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {{
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        }},
    }},
]

WSGI_APPLICATION = '{wsgi_app}'


# Database
# https://docs.djangoproject.com/en/2.0/ref/settings/#databases

DATABASES = {{
    'default': {{
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }}
}}


# Password validation
# https://docs.djangoproject.com/en/2.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {{
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    }},
    {{
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    }},
    {{
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    }},
    {{
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    }},
]


# Internationalization
# https://docs.djangoproject.com/en/2.0/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.0/howto/static-files/

STATIC_URL = '/static/'
"""


class SettingsStage(BuildStage):
    def run(self):
        root = Path(self.build.settings_pckg_path)
        root.mkdir()
        (root / '__init__.py').touch()
        settings_py = root / 'settings.py'
        content = self._get_content()
        settings_py.write_text(content)
    
    def _get_content(self):
        proj_name = self.build.project.name
        ctx = {
            'project_name': proj_name,
            'SECRET_KEY': '=8od22*=9b5d8kd!pu_tq62l3&o5@xm_l0@1uw6y&h)r==$@lu',
            'url_conf': '{}.urls'.format(proj_name),
            'wsgi_app': '{}.wsgi.application'.format(proj_name),
            'INSTALLED_APPS': self._installed_apps_lines()
        }
        result = TPL.format(**ctx)
        return result
    
    def _installed_apps_lines(self):
        result = [
            "'django.contrib.admin',",
            "'django.contrib.auth',",
            "'django.contrib.contenttypes',",
            "'django.contrib.sessions',",
            "'django.contrib.messages',",
            "'django.contrib.staticfiles',",
            "",
        ]
        for app in self.build.details['apps']:
            if not app['external']:
                result.append("'{}',".format(app['name']))
        return '\n    '.join(result)



import os
import json

USER_PLUGINS_DIR = './plugin_dir'  # TODO: mabye ~/.prototyper ?


def search_plugins(query):
    results = []
    ud = USER_PLUGINS_DIR
    if os.path.exists(ud):
        for item in os.listdir(ud):
            config = os.path.join(ud, item, 'config.json')
            if not os.path.exists(config):
                continue
            with open(config) as f:
                data = json.load(f)
                meta = {k: data.get(k, '') for k in ['name', 'title', 'version', 'description']}
                meta['url'] = os.path.join(ud, item)
                results.append(meta)
    return results



import os
import re
import json
from .initial import create_new_project


class Project(object):
    def __init__(self, path):
        self.path = os.path.abspath(path)
        self.name = self.get_name(path)
        self.storage_path = os.path.join(self.path, '.djangoprototyper')
        self.storage_file = os.path.join(self.storage_path, 'project.json')
        self.plugins_path = os.path.join(self.storage_path, 'plugins')
        self.init_storage()
    
    def init_storage(self):
        if os.path.exists(self.path) and not os.path.exists(self.storage_path):
            raise RuntimeError('Cannot init project. Path "%s" already exist and it is not djangoprototyper' % self.path)
        if not os.path.exists(self.path):
            self.init_new()
        else:
            self.load()  # pasing check
    
    def init_new(self):
        print('Creating new project', self.name)
        os.makedirs(self.storage_path)
        data = create_new_project(self.name)
        self.save(data)
    
    def load(self):
        with open(self.storage_file, 'r') as f:
            data = json.load(f)
        data['name'] = self.name
        return data
    
    def save(self, data):
        with open(self.storage_file, 'w') as f:
            json.dump(data, f, indent=1)
    
    def get_name(self, path):
        name = os.path.basename(self.path)
        return re.sub(r'[^a-z\d_]', '', name)



import sys
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 5)

if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""
==========================
Unsupported Python version
==========================
Django Prototyper requires Python {}.{}, but you're trying to
install it on Python {}.{}.
""".format(*(REQUIRED_PYTHON + CURRENT_PYTHON)))
    sys.exit(1)


here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='django-prototyper',
    version='0.1.12',
    description='Django prototyping tool',
    long_description=long_description,
    url='https://github.com/vitalik/django-prototyper',
    author='Vitaliy Kucheryaviy',
    author_email='p.p.r.vitaly@gmail.com',

    packages=find_packages(exclude=['tests']),

    install_requires=['django'],

    package_data={'prototyper': ['static/build.js', 'static/logo.png']},
    include_package_data=True,

    entry_points={
        'console_scripts': [
            'prototyper=prototyper.cli:main',
        ],
    },


    classifiers=[
        'Development Status :: 3 - Alpha',  # 3 - Alpha, 4 - Beta, 5 - Production

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='django prototype boilerplate development uml diagrams',
)



import os


def create_new_project(name):
    result = INITIAL_PROJECT.copy()
    result['settings']['ADMIN_SITE_HEADER'] = name
    result['settings']['EMAIL_SUBJECT_PREFIX'] = '[{}] '.format(name)

    if os.environ.get('PROTOTYPER_DEV') == 'yes':
        result['apps'].extend(DEMO_APPS)

    for app, models in INITIAL_DJANGO_APPS:
        result['apps'].append({
            'name': app.split('.')[-1],
            'external': True,
            'models': [{'name': m, 'fields': [], 'admin': {'generate': False}} for m in models]
        })

    return result


INITIAL_DJANGO_APPS = [
    ('django.contrib.admin', ['LogEntry']),
    ('django.contrib.auth', ['AbstractBaseUser', 'AbstractUser', 'Group', 'Permission', 'PermissionsMixin', 'User']),
    ('django.contrib.contenttypes', ['ContentType']),
    ('django.contrib.sessions', ['AbstractBaseSession', 'Session']),
    ('django.contrib.messages', []),
    ('django.contrib.staticfiles', []),
]

INITIAL_PROJECT = {
    'version': '0.1',
    'build_settings': {},
    'settings': {
        'ADMIN_SITE_HEADER': 'project1',
        'DEFAULT_FROM_EMAIL': 'noreply@site.com',
        'EMAIL_SUBJECT_PREFIX': '[project1] ',
        'LANGUAGE_CODE': 'en-us',
        'TIME_ZONE': 'Europe/Brussels',
        'USE_I18N': True,
        'USE_L10N': True,
    },
    'name': 'project1',
    'ui': {'models_view': 'designer'},
    'plugins': [],
    'apps': [],
}

DEMO_APPS = [
    {'models': [
        {'admin': {'generate': True},
         'fields': [{'attrs': {'max_length': '10'},
                     'name': 'title',
                     'relation': None,
                     'type': 'CharField'},
                    {'attrs': {},
                     'name': 'slug',
                     'relation': None,
                     'type': 'SlugField'}],
         'name': 'Category',
         'ui_left': 20,
         'ui_top': 20},
        {'admin': {'generate': True},
         'fields': [{'attrs': {},
                     'name': 'title',
                     'relation': None,
                     'type': 'CharField'},
                    {'attrs': {},
                     'name': 'slug',
                     'relation': None,
                     'type': 'SlugField'},
                    {'attrs': {},
                     'name': 'categories',
                     'relation': 'products.Category',
                     'type': 'ManyToManyField'},
                    {'attrs': {},
                     'name': 'description',
                     'relation': None,
                     'type': 'TextField'},
                    {'attrs': {'decimal_places': '2',
                               'max_digits': '10'},
                     'name': 'price',
                     'relation': None,
                     'type': 'DecimalField'}],
         'name': 'Product',
         'ui_left': 20,
         'ui_top': 104},
        {'admin': {'generate': True},
         'fields': [{'attrs': {},
                     'name': 'product',
                     'relation': 'products.Product',
                     'type': 'ForeignKey'},
                    {'attrs': {'upload_to': 'products/images/'},
                     'name': 'image',
                     'relation': None,
                     'type': 'ImageField'}],
         'name': 'Image',
         'ui_left': 20,
         'ui_top': 264}],
        'name': 'products',
        'ui_color': '#9AAF90',
        'external': False},
    {'models': [{'admin': {'generate': True},
                 'fields': [{'attrs': {},
                             'name': 'number',
                             'relation': None,
                             'type': 'CharField'},
                            {'attrs': {},
                             'name': 'timestamp',
                             'relation': None,
                             'type': 'DateTimeField'},
                            {'attrs': {'decimal_places': '2',
                                       'max_digits': '10'},
                             'name': 'total_amount',
                             'relation': None,
                             'type': 'DecimalField'}],
                 'name': 'Order',
                 'ui_left': 200,
                 'ui_top': 20},
                {'admin': {'generate': True},
                 'fields': [{'attrs': {},
                             'name': 'order',
                             'relation': 'orders.Order',
                             'type': 'ForeignKey'},
                            {'attrs': {},
                             'name': 'product',
                             'relation': 'products.Product',
                             'type': 'ForeignKey'},
                            {'attrs': {'decimal_places': '2',
                                       'max_digits': '10'},
                             'name': 'price',
                             'relation': None,
                             'type': 'DecimalField'},
                            {'attrs': {},
                             'name': 'quantity',
                             'relation': None,
                             'type': 'PositiveSmallIntegerField'}],
                 'name': 'OrderItem',
                 'ui_left': 200,
                 'ui_top': 140}],
     'name': 'orders',
     'ui_color': '#F77A5E',
     'external': False},
    {'models': [{'admin': {'generate': True},
                 'fields': [{'attrs': {},
                             'name': 'title',
                             'relation': None,
                             'type': 'CharField'},
                            {'attrs': {},
                             'name': 'slug',
                             'relation': None,
                             'type': 'SlugField'},
                            {'attrs': {},
                             'name': 'publication_date',
                             'relation': None,
                             'type': 'DateTimeField'},
                            {'attrs': {},
                             'name': 'text',
                             'relation': None,
                             'type': 'TextField'}],
                 'name': 'News',
                 'ui_left': 480,
                 'ui_top': 20}],
     'name': 'news',
     'ui_color': '#DAC9B7',
     'external': False},
    {'models': [{'admin': {'generate': True},
                 'fields': [
                            {'attrs': {},
                             'name': 'big_integer',
                             'relation': None,
                             'type': 'BigIntegerField'},
                            {'attrs': {},
                             'name': 'binary',
                             'relation': None,
                             'type': 'BinaryField'},
                            {'attrs': {},
                             'name': 'boolean',
                             'relation': None,
                             'type': 'BooleanField'},
                            {'attrs': {},
                             'name': 'char',
                             'relation': None,
                             'type': 'CharField'},
                            {'attrs': {},
                             'name': 'date',
                             'relation': None,
                             'type': 'DateField'},
                            {'attrs': {},
                             'name': 'date_time',
                             'relation': None,
                             'type': 'DateTimeField'},
                            {'attrs': {'max_digits': 5, 'decimal_places': 2},
                             'name': 'decimal',
                             'relation': None,
                             'type': 'DecimalField'},
                            {'attrs': {},
                             'name': 'duration',
                             'relation': None,
                             'type': 'DurationField'},
                            {'attrs': {},
                             'name': 'email',
                             'relation': None,
                             'type': 'EmailField'},
                            {'attrs': {'upload_to': 'products/images/'},
                             'name': 'file',
                             'relation': None,
                             'type': 'FileField'},
                            {'attrs': {},
                             'name': 'file_path',
                             'relation': None,
                             'type': 'FilePathField'},
                            {'attrs': {},
                             'name': 'float',
                             'relation': None,
                             'type': 'FloatField'},
                            {'attrs': {},
                             'name': 'foreign_key',
                             'relation': 'products.Product',
                             'type': 'ForeignKey'},
                            {'attrs': {},
                             'name': 'generic_ipaddr',
                             'relation': None,
                             'type': 'GenericIPAddressField'},
                            {'attrs': {'upload_to': 'products/images/'},
                             'name': 'image',
                             'relation': None,
                             'type': 'ImageField'},
                            {'attrs': {},
                             'name': 'integer',
                             'relation': None,
                             'type': 'IntegerField'},
                            {'attrs': {},
                             'name': 'many_to_many',
                             'relation': 'news.News',
                             'type': 'ManyToManyField'},
                            {'attrs': {},
                             'name': 'null_boolean',
                             'relation': None,
                             'type': 'NullBooleanField'},
                            {'attrs': {},
                             'name': 'one_to_one',
                             'relation': 'orders.Order',
                             'type': 'OneToOneField'},
                            {'attrs': {},
                             'name': 'positive_integer',
                             'relation': None,
                             'type': 'PositiveIntegerField'},
                            {'attrs': {},
                             'name': 'positive_small_int',
                             'relation': None,
                             'type': 'PositiveSmallIntegerField'},
                            {'attrs': {},
                             'name': 'slug',
                             'relation': None,
                             'type': 'SlugField'},
                            {'attrs': {},
                             'name': 'small_integer',
                             'relation': None,
                             'type': 'SmallIntegerField'},
                            {'attrs': {},
                             'name': 'text',
                             'relation': None,
                             'type': 'TextField'},
                            {'attrs': {},
                             'name': 'time',
                             'relation': None,
                             'type': 'TimeField'},
                            {'attrs': {},
                             'name': 'url',
                             'relation': None,
                             'type': 'URLField'},
                            {'attrs': {},
                             'name': 'uuid',
                             'relation': None,
                             'type': 'UUIDField'}],
                 'name': 'AllFields',
                 'ui_left': 480,
                 'ui_top': 160}],
     'name': 'temp',
     'ui_color': '#9CC4E4',
     'external': False},
]



import os
from django.conf import settings


def load_py_plugins():
    results = []
    path = settings.PROTOTYPER_PROJECT.plugins_path
    if not os.path.exists(path):
        return results
    
    plugins = settings.PROTOTYPER_PROJECT.load()['plugins']
    installed_plugins = set([p['name'] for p in plugins])
    
    for p in os.listdir(path):
        if p not in installed_plugins:
            continue
        plugin_module = os.path.join(path, p, 'plugin.py')
        print(plugin_module)
        if os.path.exists(plugin_module):
            klass = load(plugin_module)
            plugin = klass(p)
            results.append(klass(p))
    return results


def load(path):
    module = _load_module('plugin', path)
    return module.Plugin


def _load_module(module, path):
    import importlib.util
    spec = importlib.util.spec_from_file_location(module, path)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    return module



import shutil
import traceback
from django.conf import settings
from .base import Build, BuildStage, pipeline
from . import stages
from ..plugins import load_py_plugins


def run_build():
    build = Build(settings.PROTOTYPER_PROJECT)
    try:
        return _run_build(build)
    except Exception as e:
        build.logger.error(traceback.format_exc())
        return build


def _run_build(build):
    plugins = _init_py_plugins(build)
    pipeline(build, plugins, [
        stages.FirstStage,
        stages.AppsPackages,
        stages.SettingsStage,
        stages.UrlsStage,
        stages.WsgiStage,
        stages.ModelsStage,
        stages.AdminStage,
        stages.RequirementsStage,
    ])
    build.cleanup()
    return build


def _init_py_plugins(build):
    build.log('Loading plugins...')
    plugins = []
    for plugin in load_py_plugins():
        plugins.append(plugin)
    build.log(str(plugins))
    return plugins



import sys
import os
import argparse
import django
from django.conf import settings
from prototyper.server import django_configure, run_server


def _parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('path_to_project')
    parser.add_argument('--build', action='store_true')
    return parser.parse_args()


def build():
    from prototyper.build import run_build
    run_build()


def main():
    args = _parse_args()
    django_configure()

    from prototyper.project import Project
    settings.PROTOTYPER_PROJECT = Project(args.path_to_project)
    django.setup()

    if args.build is True:
        build()
    else:
        run_server()



import os
import shutil
import json
import zipfile
import re
from django.conf import settings


def install(name, url):
    _validate(name, url)
    plugins_path = settings.PROTOTYPER_PROJECT.plugins_path
    installer = get_installer(name, url, plugins_path)
    installer.clean()
    installer.install()
    return installer.config()


def _validate(name, url):
    # we do not allow any other characters for plugin name so that user do no play with "/etc/passwd" sort of names
    assert re.match(r'^[a-z-_]+$', name.lower()), "Invalid plugin name"


def get_installer(name, url, plugins_path):
    if os.path.exists(url):
        if url.lower().endswith('.zip'):
            return ZipFileInstaller(name, url, plugins_path)
        if os.path.isdir(url):
            return PathInstaller(name, url, plugins_path)
    elif url.lower().startswith('http'):
        if url.lower().endswith('.zip'):
            return ZipUrlInstaller(name, url, plugins_path)
        # TODO: github/git ?
    raise NotImplementedError('Do not know how to install %s, should be url/path to .zip file or path to directory')


class Installer(object):
    def __init__(self, name, url, plugins_path):
        self.name = name
        self.url = url
        # basename is important so that users do not play with unsecure names
        self.plugin_dest = os.path.join(plugins_path, os.path.basename(name))
    
    def config(self):
        with open(os.path.join(self.plugin_dest, 'config.json')) as f:
            return json.load(f)
    
    def clean(self):
        if os.path.exists(self.plugin_dest):
            shutil.rmtree(self.plugin_dest)
    
    def install(self):
        raise NotImplementedError('Please implement install')


class PathInstaller(Installer):
    def install(self):
        shutil.copytree(self.url, self.plugin_dest)


class ZipFileInstaller(Installer):
    def install(self):
        with zipfile.ZipFile(self.url, 'r') as z:
            z.extractall(self.plugin_dest)


class ZipUrlInstaller(Installer):
    def install(self):
        path = self.download()
        with zipfile.ZipExtFile(path, 'r') as z:
            z.extractall(self.plugin_dest)


# DEMO_PLUGIN = """from prototyper.plugins import PluginBase


# class Plugin(PluginBase):
    
#     def on_build_complete(self):
#         print('on_build_complete')
# """


# def demo(path):
#     plugin_py = path / 'plugin.py'
#     plugin_py.write_text(DEMO_PLUGIN)


# def _plugin_path(name):
#     path = os.path.join(settings.PROTOTYPER_PROJECT.plugins_path, name)
#     path = pathlib.Path(path)
#     path.mkdir(parents=True, exist_ok=True)
#     return path
